@@ -26,6 +26,7 @@
 #include <stdarg.h>
 #include <time.h>
 #include <termios.h>
+#include <curl/curl.h>
 #include "../list.h"
 #include "queue.h"
 
@@ -53,6 +54,40 @@ void timemods(void)
   havemods=(mods.itemcount>1); // Not counting modbot as a mod
 }
 
+struct writebuf
+{
+  char* buf;
+  int len;
+};
+
+size_t writehttp(char* ptr, size_t size, size_t nmemb, void* x)
+{
+  struct writebuf* data=x;
+  size*=nmemb;
+  data->buf=realloc(data->buf, data->len+size+1);
+  memcpy(&data->buf[data->len], ptr, size);
+  data->len+=size;
+  data->buf[data->len]=0;
+  return size;
+}
+
+char* http_get(const char* url)
+{
+  CURL* curl=curl_easy_init();
+  if(!curl){return 0;}
+  curl_easy_setopt(curl, CURLOPT_URL, url);
+  struct writebuf writebuf={0, 0};
+  curl_easy_setopt(curl, CURLOPT_WRITEFUNCTION, writehttp);
+  curl_easy_setopt(curl, CURLOPT_WRITEDATA, &writebuf);
+  curl_easy_setopt(curl, CURLOPT_FOLLOWLOCATION, 1L);
+  curl_easy_setopt(curl, CURLOPT_USERAGENT, "Mozilla Firefox");
+  char err[CURL_ERROR_SIZE];
+  curl_easy_setopt(curl, CURLOPT_ERRORBUFFER, err);
+  if(curl_easy_perform(curl)){curl_easy_cleanup(curl); printf("%s\n", err); return 0;}
+  curl_easy_cleanup(curl);
+  return writebuf.buf; // should be free()d when no longer needed
+}
+
 void say(const char* pm, const char* fmt, ...)
 {
   va_list va;
@@ -135,19 +170,19 @@ unsigned int getduration(const char* vid)
   return len;
 }
 
-unsigned int waitskip=0;
-time_t waitskiptime=0;
+time_t waitskip=0;
 void playnextvid()
 {
   waitskip=0;
   playing=queue.items[0].video;
   requester=queue.items[0].requester;
   title=queue.items[0].title;
-  say(0, "/mbs youTube %s %u %s\n", playing, queue.items[0].timeoffset, queue.items[0].title);
+  say(0, "/mbs youTube %s %u %s\n", playing, queue.items[0].timeoffset*1000, queue.items[0].title);
+  unsigned int duration=getduration(playing)-queue.items[0].timeoffset;
   --queue.itemcount;
   memmove(queue.items, &queue.items[1], sizeof(struct queueitem)*queue.itemcount);
   // Find out the video's length and schedule an alarm for then
-  alarm(getduration(playing));
+  alarm(duration);
   started=time(0);
 }
 
@@ -165,28 +200,38 @@ void playnext(int x)
     if(!waitskip)
     {
       say(0, "Next video (%s, %s) is not yet approved by mods\n", queue.items[0].video, queue.items[0].title);
+      waitskip=time(0)+120;
+      alarm(120);
+      return;
+    }else{
+      if(time(0)<waitskip){return;} // Not time yet
       unsigned int i;
       for(i=1; i<queue.itemcount; ++i)
       {
         if(list_contains(&goodvids, queue.items[i].video))
         {
-          waitskip=i;
-          waitskiptime=time(0)+120;
-          alarm(120);
+          say(0, "Skipping http://youtube.com/watch?v=%s because it is still not approved after 2 minutes\n", queue.items[0].video);
+          queue_movetofront(&queue, i);
+          waitskip=0;
           break;
         }
       }
-      return;
-    }else{
-      if(time(0)<waitskiptime){return;} // Not time yet
-      say(0, "Skipping http://youtube.com/watch?v=%s because it is still not approved after 2 minutes\n", queue.items[0].video);
-      queue_movetofront(&queue, waitskip);
-      waitskip=0;
+      if(i==queue.itemcount){return;}
     }
   }
   playnextvid();
 }
 
+void playfor(const char* nick)
+{
+  if(started)
+  {
+    say(0, "/priv %s /mbs youTube %s %u\n", nick, playing, (time(0)-started)*1000);
+  }else{
+    say(0, "/priv %s /mbsp youTube %s 0\n", nick, playing); // Start paused
+  }
+}
+
 int main(int argc, char** argv)
 {
   // Handle arguments (-d, -l, -h additions, the rest are handled by tc_client)
@@ -419,6 +464,19 @@ int main(int argc, char** argv)
             say(pm, "Video '%s' is marked as bad, won't add to queue\n", title);
             continue;
           }
+          if(!list_contains(&goodvids, vid)) // Check that this video can be embedded (for the flash client)
+          {
+            char url[strlen("https://www.youtube.com/watch?v=0")+strlen(vid)];
+            sprintf(url, "https://www.youtube.com/watch?v=%s", vid);
+            char* page=http_get(url);
+            void* res=strstr(page, "youtube.com/embed/");
+            free(page);
+            if(!res)
+            {
+              say(pm, "The video I found for '%s' was not embeddable, sorry\n", &msg[9]);
+              continue;
+            }
+          }
           if(list_contains(&mods, nick)) // Auto-approve for mods
           {
             list_add(&goodvids, vid);
@@ -437,7 +495,7 @@ int main(int argc, char** argv)
               say(pm, "Video '%s' (%s) is added to the queue but will need to be approved by a mod\n", vid, title);
             }
           }
-          else if(!playing && !waitskip){playnext(0);}
+          else if(!playing){playnext(0);}
           else{say(pm, "Added '%s' to queue, it has already been approved\n", title);}
         }
         // Undo
@@ -570,7 +628,7 @@ int main(int argc, char** argv)
           if(playing)
           {
             space[0]=0;
-            say(0, "/priv %s /mbs youTube %s %u\n", nick, playing, (time(0)-started)*1000);
+            playfor(nick);
           }else{
             say(pm, "Nothing is playing\n");
           }
@@ -718,13 +776,17 @@ int main(int argc, char** argv)
             list_save(&goodvids, "goodvids.txt");
             playnext(0);
           }
-          else if(!strncmp(msg, "/mbsk youTube ", 14)) // Seeking
+          else if((!strncmp(msg, "/mbsk youTube ", 14) || !strncmp(msg, "/mbpl youTube ", 14)) && playing) // Seeking, or resume playing
           {
             unsigned int pos=strtol(&msg[14], 0, 0)/1000;
             alarm(getduration(playing)-pos);
             started=time(0)-pos;
           }
-          // TODO: handle /mbpa (pause) and /mbpl (resume play)
+          else if(!strcmp(msg, "/mbpa youTube")) // Pause
+          {
+            alarm(0);
+            started=0;
+          }
         }
       }else{ // Actions
         if(!strncmp(space, " changed nickname to ", 21))
@@ -751,7 +813,7 @@ int main(int argc, char** argv)
           if(playing)
           {
             space[0]=0;
-            say(0, "/priv %s /mbs youTube %s %u\n", nick, playing, (time(0)-started)*1000);
+            playfor(nick);
           }
         }
         else if(!strcmp(space, " left the channel"))
